
#include <Arbol1.H>
#include <Hoja1.H>

#include <stdlib.h>

const bool debugArbol1 = false ;

double Rango::LeeValor()
{
    double r = double(random()) / double(RAND_MAX) ;
    return min+r*(max-min) ;
}

Arbol1::Arbol1()
{

   numRamif = 4 ;
   nivelMaximo = 6 ;

   radSegmento     = Rango(0.19,0.21) ;
   fRedRadSegmento = Rango(0.65,0.75) ;
   fRedAltSegmento = Rango(0.65,0.75) ;

   /** rama "primaria" (continua aproximadamente en el eje de la actual) **/

   altRamif[0]     = Rango(1.0,1.0) ;
   angDesvRamif[0] = Rango(0.05*(0.5*M_PI),0.10*(0.5*M_PI)) ;
   angRotaRamif[0] = Rango(0.0,2.0*M_PI) ;

#define V 0.1           // variabilidad en tanto por 1 del angulo de rotacion de las ramas
#define F (2.0*M_PI)    // factor de paso [0-1] ==> [0-2*PI]

   /** primera rama secundaria **/

   altRamif[1] =     Rango(0.3,0.5) ;
   angDesvRamif[1] = Rango(0.10*M_PI,0.30*M_PI) ;
   angRotaRamif[1] = Rango(((0.0/3.0)-V)*F, ((0.0/3.0)+V)*F) ;

   /** segunda rama secundaria **/

   altRamif[2] =     Rango(0.5,0.7) ;
   angDesvRamif[2] = Rango(0.10*M_PI,0.30*M_PI) ;
   angRotaRamif[2] = Rango(((1.0/3.0)-V)*F, ((1.0/3.0)+V)*F) ;

   /** tercera rama secundaria **/

   altRamif[3] =     Rango(0.5,0.7) ;
   angDesvRamif[3] = Rango(0.10*M_PI,0.30*M_PI) ;
   angRotaRamif[3] = Rango(((2.0/3.0)-V)*F, ((2.0/3.0)+V)*F) ;

   hojas = new Hoja1 ;

   if ( hojas == NULL )
   {
      std::cerr << "ohoh!" ;
      exit(1) ;
   }

}


void  Arbol1::Rama
(
    int               nivelActual ,
    const RefSys &    ejeTronco ,
    std::ostream &         os
)
{
    if ( nivelActual >= nivelMaximo )
    {
       if ( hojas != NULL )
       {
          hojas->Rama(0,ejeTronco,os) ;
       }
       return ;
    }

    int n = nivelActual + 1 ;

    if ( debugArbol1 )
    {
        std::cout << std::endl << "*** entra Rama(...)"
             << std::endl << "  nivelActual = " << nivelActual ;
    }




    bool soloEjes = false ;

    if ( soloEjes )
    {
       Vector3d p0(0,0,0),
                p1(0,0,0),
                p2(0,1,0) ;

       os << std::endl
          << std::endl << "begin mesh"
          << std::endl << "   begin vertexs"
          << std::endl << "      " << ejeTronco.PuntoLocalAGlobal(p0)
          << std::endl << "      " << ejeTronco.PuntoLocalAGlobal(p1)
          << std::endl << "      " << ejeTronco.PuntoLocalAGlobal(p2)
          << std::endl << "   end vertexs"
          << std::endl << "   begin faces"
          << std::endl << "      (0,1,2)"
          << std::endl << "   end faces"
          << std::endl << "end mesh" ;
    }
    else
    {
       GeneraMallaCilindrica(ejeTronco,os) ;
    }




    for( unsigned i=0 ; i < numRamif ; i++ )
    {
        double exz = fRedRadSegmento.LeeValor() ;

        RefSys rs = ejeTronco
                    .DesplRotarY
                    (
                         altRamif[i].LeeValor(),
                         angDesvRamif[i].LeeValor(),
                         angRotaRamif[i].LeeValor()
                    )
                    .Escalar
                    (
                        exz, fRedAltSegmento.LeeValor(), exz
                    ) ;

        Rama(n,rs,os) ;
    }

}

Vector3d Arbol1::EvaluaCilindro
(
    const RefSys & eje ,
    double         u ,
    double         v
)
    const
{
    double uu = 2.0*M_PI*u ;
    double r = 0.03 ;

    Vector3d p( r*cos(uu),v, r*sin(uu) ) ;

    return eje.PuntoLocalAGlobal( p ) ;
}

void Arbol1::GeneraMallaCilindrica
(
    const RefSys & eje ,
    std::ostream &      os

)
    const
{

   unsigned long int nu, nv ;

   // calcula nu, nv

   double alt = eje.LeeEje(Y).Module() ;
   double rad = eje.LeeEje(X).Module() ;

   nv = unsigned ( double(alt) * double(10) ) ;
   nu = unsigned ( double(0.03*rad*2.0*M_PI) * double(40) ) ;

   if ( nv < 2 ) nv = 2 ;
   if ( nu < 3 ) nu = 3 ;

#define INDEX(iu,iv) (((iv)*(nu+1))+(iu))

   unsigned long i,iu, iv, ntotal ;

   double du = 1.0/(double)nu ,
          dv = 1.0/(double)nv ;

   ntotal = (nu+1)*(nv+1) ;

   os << std::endl
      << std::endl << "// tronco: |Y| = " << eje.LeeEje(Y).Module()
      << std::endl << "//         |X| = " << eje.LeeEje(X).Module() << " |Z| = " << eje.LeeEje(Z).Module() ;


   os << std::endl << "begin mesh"
      << std::endl << "   begin vertexs" ;

   for( i = 0 ; i < ntotal ; i++ )
   {

      double    u,v ;
      Vector3d  p00, p01, p10, d0, d1, n ;
      unsigned  iu,iv ;

      double    dx0,dy0,dz0,
                dx1,dy1,dz1,
                nx,ny,nz,
                mn ;

      iu = i % (nu+1) ;
      iv = i / (nu+1) ;

      u = (double)iu / (double)nu ;
      v = (double)iv / (double)nv ;

      p00 = EvaluaCilindro( eje, u,    v    ) ;
      p01 = EvaluaCilindro( eje, u,    v+dv ) ;
      p10 = EvaluaCilindro( eje, u+du, v    ) ;

      d0 = p01 - p00 ;
      d1 = p10 - p00 ;

      n = d0*d1 ;

      os << std::endl << "        normal " << (n.Normalized())
         << std::endl << "        " << p00 ;

   }


   os << std::endl << "   end vertexs"
      << std::endl << "   begin faces"
      << std::endl << "         rgb ( 1.0,0.5,0.0) " ;

   /**
   if ( color_malla )
       printf("\n      rgb (%f,%f,%f)",(float)mr,(float)mg,(float)mb);

   color_malla = 0 ;
   **/


   for( iu=0 ; iu < nu ; iu++ )
   for( iv=0 ; iv < nv ; iv++ )
   {
      unsigned
         i00 = INDEX(iu  ,iv  ) ,
         i01 = INDEX(iu  ,iv+1) ,
         i10 = INDEX(iu+1,iv  ) ,
         i11 = INDEX(iu+1,iv+1) ;

      os << std::endl << "      (" << i00 << "," << i01 << "," << i10 << ")"
         << std::endl << "      (" << i10 << "," << i01 << "," << i11 << ")" ;

   }

   os << std::endl << "   end faces"
      << std::endl << "end mesh"
      << std::endl ;

}






